<?php declare(strict_types=1);

namespace Dump;

class Dump {
    /**
     * Latest stack trace found from the debugger
     * @var
     */
    protected $trace = null;

    /**
    * @var mixed
    */
    protected $content = null;

    public function __construct(...$content) {
        $this->trace = (debug_backtrace(DEBUG_BACKTRACE_IGNORE_ARGS, 3)[2] ?? [
            "line" => -1,
            "file" => "unknown",
        ]) + [ 'time' => round(1000 * (microtime(true) - $_SERVER["REQUEST_TIME_FLOAT"]), 4) ];

        $this->content = $content;
    }

    public function renderArray() {
        return [
            'content' => $this->arrayKeysToTypes($this->content),
            'file' => $this->trace['file'] ?? '?',
            'line' => $this->trace['line'] ?? '?',
            'time' => $this->trace['time'],
        ];
    }

    public function renderHtml() {
        $data = $this->getDumpContent();

        $this->trace['file'] ??= "?";
        $this->trace['line'] ??= "?";
        
        return <<<HTML
<pre style='-webkit-overflow-scrolling: touch;background-color: #f5f5f5;color: #4a4a4a;font-size: 12px;line-height:18px;overflow-x: auto;padding: 0.5rem 1.5rem;word-wrap: normal;white-space: pre;margin:0.33rem 0;position: relative;z-index:9999999999;'>
<div style='color:#9f9f9f;font-size:10px;' title='{$this->trace['file']}'>[ file: {$this->trace['file']}:{$this->trace['line']} ] <small>timestamp: {$this->trace['time']}</small></div> 
$data
</pre>
HTML;
    }

    public function renderText() {
        $data = $this->getDumpContent();

        $this->trace['file'] ??= "?";
        $this->trace['line'] ??= "?";

        $header = "File: {$this->trace['file']} | line : {$this->trace['line']} | time : {$this->trace['time']}";

        return str_repeat('-', strlen($header)) . PHP_EOL . $header . PHP_EOL . str_repeat('-', strlen($header)) . PHP_EOL . $data . PHP_EOL;
    }

    protected function getDumpContent() {
        ob_start();
        var_dump(...($this->content !== [] ? $this->content : [null]));
        return ob_get_clean();
    }

    protected function arrayKeysToTypes(array $content) : array
    {
        $newKeys = [];

        foreach($content as $key => $obj) {
            $newKeys[] = sprintf("%s-%s", $key, $this->typeAsKey($obj));
        }

        return array_combine($newKeys, $content);
    }

    protected function typeAsKey(mixed $obj) : string
    {
        if (is_object($obj)) {
            return $obj::class;
        }
        elseif (is_array($obj)) {
            return "array";
        }
        elseif (is_string($obj)) {
            return "string";
        }
        elseif (is_int($obj)) {
            return "integer";
        }
        elseif (is_float($obj)) {
            return "float";
        }
        elseif(is_callable($obj)) {
            return "callable";
        }
        elseif(is_bool($obj)) {
            return "boolean";
        }
        elseif(is_double($obj)){
            return "double";
        }
        elseif(is_long($obj)){
            return "long";
        }
        elseif(is_iterable($obj)) {
            return "iterable";
        }

        return gettype($obj);
    }
}
